#!/usr/bin/perl -w

# Copyright 2015, 2018, 2020 Kevin Ryde.
#
# This file is part of x2gpm.
#
# x2gpm is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# x2gpm is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
# details.
#
# You should have received a copy of the GNU General Public License along
# with X2gpm; see the file COPYING.  Failing that, goto
# <http://www.gnu.org/licenses/>. */


use strict;
use FindBin;
use File::Slurp;
use File::stat;
use IPC::Run;

my $program_filename = File::Spec->catfile($FindBin::Bin,
                                           $FindBin::Script);
chdir '/tmp' or die;

File::Slurp::write_file('favicon.tex',<<'HERE');
\documentclass[a4paper]{article}
\pagestyle{empty}
\usepackage{tikz}
\begin{document}
\begin{tikzpicture}[scale=.22]
  \node at (0,0) [inner sep=0pt] (X2) {\textbf{X2}};
  \node at (X2.south) [below=.6ex,inner sep=0pt] {\textbf{gpm}};
\end{tikzpicture}
\end{document}
HERE

IPC::Run::run(['latexmk','-gg','-pdf','favicon.tex'], '<',File::Spec->devnull)
  or die;

IPC::Run::run(['pdfcrop',
               '--verbose',
               '--margins','1',
               'favicon.pdf',
               'favicon-cropped.pdf'],
              '<', File::Spec->devnull)
  or die;
rename 'favicon-cropped.pdf', 'favicon.pdf' or die;

# IPC::Run::run(['pdf90','favicon.pdf',
#                '--outfile','favicon-cropped.pdf']);
# rename 'favicon-cropped.pdf', 'favicon.pdf' or die;

# IPC::Run::run(['xpdf','favicon.pdf']);

IPC::Run::run(['gs',
               '-q', # quiet, cf -dQUIET
               '-dNOPAUSE',
               '-g16x16',
               '-sDEVICE=pnggray',
               '-dPDFFitPage',
               # '-dEPSFitPage',
               '-dORIENT=0',
               "-dTextAlphaBits=4",     # grey scale edge blurring
               "-dGraphicsAlphaBits=4",
               "-sOutputFile=favicon.png",
               'favicon.pdf'],
              '<', File::Spec->devnull)
  or die;
print "\n";

IPC::Run::run(['convert',
               '-rotate','90',
               '-strip',
               'favicon.png','favicon-rotated.png']);
rename 'favicon-rotated.png', 'favicon.png' or die;

# PNG spec 11.3.4.2 suggests RFC822 (or rather RFC1123) for CreationTime
use constant STRFTIME_FORMAT_RFC822 => '%a, %d %b %Y %H:%M:%S %z';
my $stat = File::stat::stat($program_filename)
  or die "Oops, cannot stat $program_filename: $!";
my $creation_time = POSIX::strftime(STRFTIME_FORMAT_RFC822,
                                    localtime($stat->mtime));

my $copyright = File::Slurp::read_file($program_filename);
$copyright =~ /^# (Copyright.*)/m or die;
$copyright = $1;

pngtextadd('Author',    'Kevin Ryde');
pngtextadd('Generator', "tools/$FindBin::Script");
pngtextadd('Title',     'X2gpm - icon');
pngtextadd('Creation Time', $creation_time);
pngtextadd('Copyright', $copyright);
pngtextadd('Homepage',  'http://user42.tuxfamily.org/x2gpm/index.html');

print "uncompressed ",-s 'favicon.png',"\n";

# no space saved by this ...
# IPC::Run::run(['optipng','-quiet','-o5','favicon.png']);
# print "compressed   ",-s 'favicon.png',"\n";

# IPC::Run::run(['xzgv','favicon.png']);

exit 0;

# add text to the png image in 'favicon.png'
sub pngtextadd {
  my ($keyword, $value) = @_;
  system('pngtextadd', "--keyword=$keyword", "--text=$value", 'favicon.png') == 0
    or die "system(pngtextadd)";
}
